# coding: utf-8
from contextlib import contextmanager
from functools  import wraps
import types


@contextmanager
def django12_debug_cursor(conn):
    """
    Monkey patch Django 1.2 to make its database debug cursor behave like
    Django 1.3 and later.

    In Django 1.2, debug cursors are enabled *only* when ``DEBUG = True``. In
    later versions they can also be enabled via
    ``BaseDatabaseWrapper.use_debug_cursor = True``. This is a good approach
    because it avoids possible side effects caused by ``DEBUG = True``.
    """
    original = conn.cursor

    @wraps(original)
    def patched(self):
        from django.conf import settings
        cursor = self._cursor()
        if (self.use_debug_cursor or
            (self.use_debug_cursor is None and settings.DEBUG)):
            return self.make_debug_cursor(cursor)
        return cursor

    try:
        conn.use_debug_cursor = None
        conn.cursor = types.MethodType(patched, conn)
        yield
    finally:
        conn.cursor = original
        del conn.use_debug_cursor



# coding: utf-8
from contextlib import contextmanager
from functools  import wraps
import sys


__all__ = ("contextdecorator", )


def contextdecorator(yielder):
    """
    Given a function that returns a contextmanager style generator, return an
    object that can be used as a context for an Attest collection, or used to
    decorate an individual test.

    Usage::

        # Define the context/decorator
        @contextdecorator
        def database(port=5432):
            try:
                db.open(port)
                yield
            finally:
                db.close()

        # Can decorate functions...
        @database
        def foo():
            ...

        # and supports keyword arguments...
        @database(port=6543)
        def foo():
            ...

        # but can still be used with contextlib.contextmanager
        with contextmanager(database)():
            ...
        with contextmanager(database(port=6543))():
            ...

        # which means it can be used as a test collection context
        suite = Tests()
        suite.context(database)
        suite.context(database(port=6532))

    .. note::

        *yielder* arguments must be optional keywords.

    """
    @wraps(yielder)
    def thing(func=None, **kwargs):
        def wrap(func):
            # wrap func using yielder as a context manager
            @wraps(func)
            def wrapped():
                with contextmanager(yielder)(**kwargs):
                    return func()
            return wrapped

        if func:
            # @thing
            # def foo():
            #     ...
            return wrap(func)

        elif kwargs:
            @wraps(yielder)
            def wrapped(func=None):
                if func:
                    # @thing(foo='bar')
                    # def foo():
                    #     ...
                    return wrap(func)
                else:
                    # contextlib.contextmanager(thing(foo='bar'))
                    return yielder(**kwargs)
            return wrapped

        else:
            # contextlib.contextmanager(thing)
            return yielder()
    return thing


@contextmanager
def nested(*managers):
    exc = None, None, None
    args = []
    exits = []
    try:
        for manager in managers:
            args.append(manager.__enter__())
            exits.append(manager.__exit__)
        yield args
    except:
        exc = sys.exc_info()
    finally:
        for exit in reversed(exits):
            try:
                if exit(*exc):
                    exc = None, None, None
            except:
                exc = sys.exc_info()
        if exc != (None, None, None):
            raise exc[0], exc[1], exc[2]



# coding: utf-8
from django.test import Client, TestCase, TransactionTestCase
from .utils import contextdecorator


__all__ = ("TransactionTestContext", "TestContext")


class TransactionTestContext(TransactionTestCase):
    __name__ = 'blah_bada_boom'

    def __init__(self, fixtures=None, urls=None, client_class=None,
                 multi_db=None):
        if fixtures:
            self.fixtures = fixtures
        if urls:
            self.urls = urls
        if client_class:
            self.client_class = client_class
        if multi_db:
            self.multi_db = multi_db

    def __call__(self):
        """
        Wrapper around default __call__ method to perform common Django
        test set up. This means that user-defined Test Cases aren't required to
        include a call to super().setUp().
        """
        self._pre_setup()
        try:
            yield getattr(self, "client_class", Client)()
        finally:
            self._post_teardown()


class TestContext(TestCase, TransactionTestContext):
    """
    Does basically the same as TransactionTestContext, but surrounds every
    test with a transaction, monkey-patches the real transaction management
    routines to do nothing, and rollsback the test transaction at the end of
    the test. You have to use TransactionTestContext, if you need transaction
    management inside a test.
    """


@contextdecorator
def database(fixtures=None):
    pass


@contextdecorator
def fixtures():
    pass





# coding: utf-8
from attest import assert_hook
from contextlib import contextmanager
from django.core.signals import request_started
from django.db import connections, DEFAULT_DB_ALIAS, reset_queries
import urlparse
from . import hacks, utils


__all__ = ("redirects", "queries")


def redirects(response, url=None, scheme=None, domain=None, port=None,
              path=None, query=None, fragment=None):
    """
    Given a Django response, asserts that it redirects to another URL, and that
    URL has various characteristics (e.g. response.path == "/foo").
    """
    assert response.status_code == 302
    if url:
        assert response["Location"] == url
    parts = urlparse.urlsplit(response["Location"])
    if scheme:
        assert parts.scheme == scheme
    if domain:
        assert parts.hostname == domain
    if port:
        assert parts.port == port
    if path:
        assert parts.path == path
    if query:
        assert parts.query == query
    if fragment:
        assert parts.fragment == fragment
    return True


@contextmanager
def queries(count=None, using=None):
    """
    A context manager that captures the queries that were made.

    :param count: assert this number of queries were made
    :param using: alias of the database to monitor

    .. note:: The `list` of queries is not populated until after the context
              manager exits.

    Usage::

        with queries() as qs:
            User.objects.count()
        assert len(qs) == 5

        # The same could be rewritten as
        with queries(count=5):
            User.objects.count()

    """
    if using is None:
        using = DEFAULT_DB_ALIAS
    conn = connections[using]

    # For compatbility with Django 1.2, apply necessary patching.
    patches = []
    if not hasattr(conn, "use_debug_cursor"):
        patches.append(hacks.django12_debug_cursor(conn))

    with utils.nested(*patches):
        # A debug cursor saves all the queries to conn.queries, in case one isn't
        # already being used, restore the current state after the test.
        was_debug_cursor = conn.use_debug_cursor
        conn.use_debug_cursor = True
        prior = len(conn.queries)
        executed = []
        request_started.disconnect(reset_queries)
        try:
            yield executed
        finally:
            request_started.connect(reset_queries)
            conn.use_debug_cursor = was_debug_cursor
        executed[:] = conn.queries[prior:]
        if count is not None:
            assert len(executed) == count



from .environment import testing_environment
from attest import reporters, Tests
from django.utils.functional import wraps


__all__ = ("AbstractReporter", "PlainReporter", "FancyReporter",
           "auto_reporter", "XmlReporter", "QuickFixReporter")


def _test_loader_factory(reporter):
    class Loader(object):
        def loadTestsFromNames(self, names, module=None):
            with testing_environment():
                Tests(names).run(reporter)
            raise SystemExit
    return Loader()


class ReporterMixin(object):
    """
    Patch :meth:`test_loader` to perform Django environment configuration.
    """
    @classmethod
    def test_loader(cls):
        return _test_loader_factory(cls)


AbstractReporter = type("AbstractReporter", (ReporterMixin, reporters.AbstractReporter), {})
PlainReporter    = type("PlainReporter",    (ReporterMixin, reporters.PlainReporter), {})
FancyReporter    = type("FancyRepoter",     (ReporterMixin, reporters.FancyReporter), {})
XmlReporter      = type("XmlReporter",      (ReporterMixin, reporters.XmlReporter), {})
QuickFixReporter = type("QuickFixReporter", (ReporterMixin, reporters.QuickFixReporter), {})


@wraps(reporters.auto_reporter)
def auto_reporter(**opts):
    return reporters.auto_reporter(**opts)

auto_reporter.test_loader = lambda: _test_loader_factory(auto_reporter)



# coding: utf8
from setuptools import setup


setup(
    name='django-attest',
    version='0.5.3',
    description='Provides Django specific testing helpers to Attest',

    author='Bradley Ayers',
    author_email='bradley.ayers@gmail.com',
    license='Simplified BSD',
    url='https://github.com/bradleyayers/django-attest/',

    packages=['django_attest'],
    install_requires=['Django >=1.1', 'Attest >=0.5', 'distribute'],

    test_loader='tests:loader',
    test_suite='tests.suite',

    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



from attest import AssertImportHook, COMPILES_AST
from contextlib import contextmanager
import django
from django.test.simple import DjangoTestSuiteRunner
import logging
from pkg_resources import parse_version


logger = logging.getLogger("django_attest")


if parse_version(django.get_version()) < parse_version('1.4'):
    logger.info("Django <1.4 has broken import infrastructure, Attest's"
                " assert hook will be disabled.")
    AssertImportHook.disable()
elif COMPILES_AST:
    AssertImportHook.enable()


@contextmanager
def testing_environment():
    """
    Context manager to put Django into a state suitable for testing.
    """
    # setup test environment
    runner = DjangoTestSuiteRunner()
    runner.setup_test_environment()
    old_config = runner.setup_databases()

    # do stuff
    yield

    # tear down environment
    runner.teardown_databases(old_config)
    runner.teardown_test_environment()

